#include "../host/inc/one_layer_nn.h"
#include "../host/inc/theta1_weights.h"
#include "../host/inc/theta2_weights.h"

#ifndef SIMD_WORK_ITEMS
#define SIMD_WORK_ITEMS 2 // must be a power of two, and divisible with number of kernels
#endif

#ifndef NUM_COMPUTE_UNITS
#define NUM_COMPUTE_UNITS 2
#endif

#define SIGMOID(x) (1.0f / (1 + exp(-x)))

__kernel
//__attribute((reqd_work_group_size(25,1,1)))
__attribute((max_work_group_size(25)))
//__attribute((num_simd_work_items(SIMD_WORK_ITEMS))) // Not possible here since work group size must be divisible and a power of 2
__attribute((num_compute_units(NUM_COMPUTE_UNITS)))
void one_layer_nn_hidden_layer(
                 // Input and output matrices
                 __global float *restrict Y,
                 __global float *restrict X)
{
    // Global ID index (offset within the NDRange)
    int global_id = get_global_id(0);

    // Local ID index (offset within a work group)
    //int local_id = get_local_id(0);

    float running_sum = 0.0f;

    for (int k = 0; k < theta1_x; ++k)
    {
        running_sum += Theta1[global_id][k] * X[k];
    }
    barrier(CLK_LOCAL_MEM_FENCE);

    // Store result in matrix Y
    Y[global_id] = SIGMOID(running_sum);
}

__kernel
//__attribute((reqd_work_group_size(10,1,1)))
__attribute((max_work_group_size(10)))
//__attribute((num_simd_work_items(SIMD_WORK_ITEMS)))
__attribute((num_compute_units(NUM_COMPUTE_UNITS)))
void one_layer_nn_output_layer(
                 // Input and output matrices
                 __global float *restrict Y,
                 __global float *restrict X)
{
    // Global ID index (offset within the NDRange)
    int global_id = get_global_id(0);

    // Local ID index (offset within a work group)
    //int local_id = get_local_id(0);

    float running_sum = 0.0f;

    running_sum += Theta2[global_id][0];

    for (int k = 1; k < theta2_x; ++k)
    {
        running_sum += Theta2[global_id][k] * X[k-1];
    }
    barrier(CLK_LOCAL_MEM_FENCE);

    // Store result in matrix Y
    Y[global_id] = SIGMOID(running_sum);
}
